class CollectionBuilder
  extend RepositoriesProxy

  ##
  # receive a list of ids and return a list of collections
  def self.build(objects = [])
    lo = []
    objects.each do |object|
      unless object['rid'].blank?
        o = Rails.cache.fetch(cache_key(object['rid'], object['last_modified']))
        o = collection_repository.find object['rid'] if o.nil?
        lo << o
      end
    end
    lo
  end

  def self.build_from_orientdb(args = {})
    lo = nil
    unless args.nil?
      # cache object when build
      lo = Rails.cache.fetch(cache_key(args['@rid'], args['last_modified']), expires_in: 12.hours) do
        obj = Collection.new(
            learning_objects: LearningObjectBuilder.build(map_rid_to_object(args['learning_objects']) || []),
            privacy: args['privacy'],
            name: args['name'],
            id: args['@rid']
        )
        obj.created_at = DateTime.strptime(args['created_at'], "%Y-%m-%d %H:%M:%S") unless args['created_at'].nil?
        obj.last_modified = DateTime.strptime(args['last_modified'], "%Y-%m-%d %H:%M:%S") unless args['last_modified'].nil?
        obj
      end
    end
    lo
  end

  def self.cache_key(rid, last_modified)
    rid + '/' + last_modified
  end

  def self.map_rid_to_object(ids)
    return {'rid' => ids} if ids.class == String

    ids.map do |id|
      {'rid' => id}
    end
  end
end
