module OrientDb
  class CollectionRepository < Base
    include OrientDb::Methods::EdgeMethods
    include OrientDb::Methods::FinderMethods

    def build_object(args={})
      CollectionBuilder.build_from_orientdb args
    end

    def save_learning_objects(collection = Collection.new)
      if collection.id.to_s.empty?
        raise NotPersistedRecordError, 'Before save learning objects to collection, you must persist using CollectionRepository::create method'
      end

      connection.command sprintf('update %s REMOVE learning_objects', collection.id)
      connection.command sprintf("update %s ADD learning_objects = [%s] SET last_modified='#{Time.now()}'", collection.id, collection.learning_objects.map(&:id).join(','))
    end

    def get_number_of_collections
      get_number_of odb_class
    end

    def create_bookmarks_collection(user)
      bookmarks = Bookmarks.new(owner: user)
      create bookmarks
    end

    def destroy_bookmarks_collection(user)
      bookmarks = user.bookmarks
      if has_bookmarks? user
        destroy bookmarks
      end
    end

    def create(object)
      super(object)
      create_edge "BelongsTo", object.id, object.owner.rid
      object
    end

    def destroy(object)
      super(object)
      destroy_edge "BelongsTo", object.id, object.owner.rid
    end

    ##
    # Select all collections in context
    def all(context = Collections::PublicContext.new)
      query = sprintf("select * from %s where name<>'Bookmarks'", context.from)
      query += sprintf("and privacy = '%s'", context.privacy) unless context.privacy.blank?
      build_objects connection.query(query)
    end

    def find_by_learning_object(learning_object)
      if learning_object.id.to_s.empty?
        return []
      end

      query = sprintf("select * from %s where learning_objects contains %s", odb_class, learning_object.id)
      build_objects connection.query(query)
    end

    def bookmarks(user)
      query = sprintf("select * from (select expand(in('BelongsTo')) from %s) where name='Bookmarks'", user.rid)
      result = build_objects(connection.query(query))
      if !result.empty?
        return result.first
      end
      create_bookmarks_collection(user)
    end

    def has_bookmarks?(user)
      if !user.bookmarks.id.nil? && !user.rid.nil?
        edge_exists? 'BelongsTo', user.bookmarks.id, user.rid
      end
    end

    def build_hash(object)
      hash = super(object)
      hash.delete("owner") # delete owner, because it represents an edge in schema
      hash.merge('name' => object.name) # forces name property for bookmarks
    end

    private

    def odb_class
      'Collection'
    end

  end
end
