module SearchEngine
  module DspaceSolr

    def search(qry)

      queryParams = merge_search_parameters(
                      {:q => '*:*', :fq => qry},
                      solr_search_default_parameters
                    )
      results = solr_client.get 'select', :params => queryParams
      objectsFound = select_items_from_results(results)

      return objectsFound
    end

    private

    #Resource types:
    # 2 = Item
    # 3 = Collection
    # 4 = Community
    def select_items_from_results(results)
      resultList = Set.new
      results["response"]["docs"].select { |r|
        r["search.resourcetype"].to_i.equal? 2
      }.each do |i|
        resultList.add i
      end
      return resultList
    end

    def solr_client
      return RSolr.connect :url => 'http://portalmecdev.c3sl.ufpr.br:8080/solr/search/'
    end

    def merge_search_parameters(p1, p2)
      return p1.merge(p2) { |key,oldval,newval| key = [newval,oldval].flatten }
    end

    def solr_search_default_parameters
      return {
          "fl"=>["
          search.resourcetype, search.resourceid,
          handle, author, title, dc.description
        "],
          "type"=>"0",
          "start"=>"0",
          "rows"=>"1000",
          "facet"=>"true",
          "facet.offset"=>"0",
          "facet.mincount"=>"1",
          "f.subject_tax_0_filter.facet.sort"=>"count",
          "f.subject_tax_0_filter.facet.limit"=>"11",
          "f.author_filter.facet.limit"=>"11",
          "f.dateIssued.year.facet.sort"=>"index",
          "f.dateIssued.year.facet.limit"=>"10",
          "f.author_filter.facet.sort"=>"count",
          "facet.field"=>["author_filter","subject_tax_0_filter","dateIssued.year"],
          "fq"=>["NOT(withdrawn:true)","NOT(discoverable:false)","read:(g0 OR g0)"]
      }
    end

  end
end
